@@ -1,35 +1,34 @@
 """image input/output functionalities."""
-
-__author__      = "Pawel Markiewicz"
-__copyright__   = "Copyright 2018"
-#-------------------------------------------------------------------------------
-
-
-import sys
-import os
-import nibabel as nib
-import pydicom as dcm
-import numpy as np
 import datetime
+import glob
+import logging
+import os
 import re
 import shutil
-import glob
+import sys
+from subprocess import run
+from textwrap import dedent
 
-from subprocess import call
+import nibabel as nib
+import numpy as np
+import pydicom as dcm
 
 #> NiftyPET resources
-import resources as rs
+from .. import resources as rs
+
+__author__      = ("Pawel J. Markiewicz", "Casper O. da Costa-Luis")
+__copyright__   = "Copyright 2020"
+log = logging.getLogger(__name__)
 
 # possible extentions for DICOM files
 dcmext = ('dcm', 'DCM', 'ima', 'IMA')
 
 
-#---------------------------------------------------------------
 def create_dir(pth):
-    if not os.path.exists(pth):    
+    if not os.path.exists(pth):
         os.makedirs(pth)
 
-#---------------------------------------------------------------
+
 def time_stamp(simple_ascii=False):
     now    = datetime.datetime.now()
     if simple_ascii:
@@ -38,25 +37,25 @@ def time_stamp(simple_ascii=False):
         nowstr = str(now.year)+'-'+str(now.month)+'-'+str(now.day)+' '+str(now.hour)+':'+str(now.minute)
     return nowstr
 
-#---------------------------------------------------------------
-def fwhm2sig (fwhm, voxsize=2.0):
+
+def fwhm2sig(fwhm, voxsize=2.0):
     return (fwhm/voxsize) / (2*(2*np.log(2))**.5)
 
 
-#================================================================================
 def getnii(fim, nan_replace=None, output='image'):
-    '''Get PET image from NIfTI file.
-    fim: input file name for the nifty image
-    nan_replace:    the value to be used for replacing the NaNs in the image.
-                    by default no change (None).
-    output: option for choosing output: image, affine matrix or a dictionary with all info.
-    ----------
+    '''
+    Get PET image from NIfTI file.
+    Arguments:
+        fim: input file name for the nifty image
+        nan_replace: the value to be used for replacing the NaNs in the image.
+                     by default no change (None).
+        output: option for choosing output: image, affine matrix or
+                a dictionary with all info.
     Return:
         'image': outputs just an image (4D or 3D)
         'affine': outputs just the affine matrix
         'all': outputs all as a dictionary
     '''
-
     import numbers
 
     nim = nib.load(fim)
@@ -65,25 +64,35 @@ def getnii(fim, nan_replace=None, output='image'):
     dimno = dim[0]
 
     if output=='image' or output=='all':
-        imr = nim.get_data()
+        imr = np.asanyarray(nim.dataobj)
         # replace NaNs if requested
-        if isinstance(nan_replace, numbers.Number): imr[np.isnan(imr)]
+        if isinstance(nan_replace, numbers.Number):
+            imr[np.isnan(imr)] = nan_replace
 
         imr = np.squeeze(imr)
         if dimno!=imr.ndim and dimno==4:
             dimno = imr.ndim
-        
+
         #> get orientations from the affine
-        ornt = nib.orientations.axcodes2ornt(nib.aff2axcodes(nim.affine))
-        trnsp = tuple(np.int8(ornt[::-1,0]))
+        ornt = nib.io_orientation(nim.affine)
+        trnsp = tuple(2-np.int8(ornt[:,0]))
         flip  = tuple(np.int8(ornt[:,1]))
 
+        #> voxel size
+        voxsize = nim.header.get('pixdim')[1:nim.header.get('dim')[0]+1]
+        #> rearrange voxel size according to the orientation
+        voxsize = voxsize[np.array(trnsp)]
+
+        #> dimensions
+        dims = dim[1:nim.header.get('dim')[0]+1]
+        dims = dims[np.array(trnsp)]
+
         #> flip y-axis and z-axis and then transpose.  Depends if dynamic (4 dimensions) or static (3 dimensions)
         if dimno==4:
             imr = np.transpose(imr[::-flip[0],::-flip[1],::-flip[2],:], (3,)+trnsp)
         elif  dimno==3:
             imr = np.transpose(imr[::-flip[0],::-flip[1],::-flip[2]], trnsp)
-    
+
     if output=='affine' or output=='all':
         # A = nim.get_sform()
         # if not A[:3,:3].any():
@@ -97,6 +106,8 @@ def getnii(fim, nan_replace=None, output='image'):
                 'dtype':nim.get_data_dtype(),
                 'shape':imr.shape,
                 'hdr':nim.header,
+                'voxsize':voxsize,
+                'dims':dims,
                 'transpose':trnsp,
                 'flip':flip}
     elif output=='image':
@@ -108,19 +119,17 @@ def getnii(fim, nan_replace=None, output='image'):
 
     return out
 
+
 def getnii_descr(fim):
-    '''
-    Extracts the custom description header field to dictionary
-    '''
+    '''Extracts the custom description header field to dictionary'''
     nim = nib.load(fim)
     hdr = nim.header
     rcnlst = hdr['descrip'].item().split(';')
     rcndic = {}
-    
+
     if rcnlst[0]=='':
-        # print 'w> no description in the NIfTI header'
         return rcndic
-    
+
     for ci in range(len(rcnlst)):
         tmp = rcnlst[ci].split('=')
         rcndic[tmp[0]] = tmp[1]
@@ -128,8 +137,8 @@ def getnii_descr(fim):
 
 
 def array2nii(im, A, fnii, descrip='', trnsp=(), flip=(), storage_as=[]):
-    '''Store the numpy array 'im' to a NIfTI file 'fnii'.
-    ----
+    '''
+    Store the numpy array 'im' to a NIfTI file 'fnii'.
     Arguments:
         'im':       image to be stored in NIfTI
         'A':        affine transformation
@@ -137,7 +146,7 @@ def array2nii(im, A, fnii, descrip='', trnsp=(), flip=(), storage_as=[]):
         'descrip':  the description given to the file
         'trsnp':    transpose/permute the dimensions.
                     In NIfTI it has to be in this order: [x,y,z,t,...])
-        'flip':     flip tupple for flipping the direction of x,y,z axes. 
+        'flip':     flip tupple for flipping the direction of x,y,z axes.
                     (1: no flip, -1: flip)
         'storage_as': uses the flip and displacement as given by the following
                     NifTI dictionary, obtained using
@@ -161,7 +170,7 @@ def array2nii(im, A, fnii, descrip='', trnsp=(), flip=(), storage_as=[]):
                  storage_as['transpose'].index(2))
 
         flip = storage_as['flip']
-    
+
 
     if trnsp==():
         im = im.transpose()
@@ -171,7 +180,7 @@ def array2nii(im, A, fnii, descrip='', trnsp=(), flip=(), storage_as=[]):
         im = im.transpose(trnsp)
     else:
         im = im.transpose(trnsp)
-    
+
 
     #> perform flip of x,y,z axes after transposition into proper NIfTI order
     if flip!=() and len(flip)==3:
@@ -187,20 +196,27 @@ def array2nii(im, A, fnii, descrip='', trnsp=(), flip=(), storage_as=[]):
     nib.save(nii, fnii)
 
 
-
-def orientnii(imfile):
+def orientnii(imfile, Cnt=None):
     '''Get the orientation from NIfTI sform.  Not fully functional yet.'''
+
+    #> check if the dictionary of constant is given
+    if Cnt is None:
+        Cnt = {}
+
     strorient = ['L-R', 'S-I', 'A-P']
     niiorient = []
     niixyz = np.zeros(3,dtype=np.int8)
     if os.path.isfile(imfile):
         nim = nib.load(imfile)
-        pct = nim.get_data()
+        pct = np.asanyarray(nim.dataobj)
         A = nim.get_sform()
         for i in range(3):
             niixyz[i] = np.argmax(abs(A[i,:-1]))
             niiorient.append( strorient[ niixyz[i] ] )
-        print niiorient
+        log.info('NIfTI orientation:\n{}'.format(niiorient))
+
+    return niiorient
+
 
 def nii_ugzip(imfile, outpath=''):
     '''Uncompress *.gz file'''
@@ -217,6 +233,7 @@ def nii_ugzip(imfile, outpath=''):
         f.write(s)
     return fout
 
+
 def nii_gzip(imfile, outpath=''):
     '''Compress *.gz file'''
     import gzip
@@ -231,12 +248,10 @@ def nii_gzip(imfile, outpath=''):
     with gzip.open(fout, 'wb') as f:
         f.write(d)
     return fout
-#===============================================================================
 
 
 def pick_t1w(mri):
-    ''' Pick the MR T1w from the dictionary for MR->PET registration.
-    '''
+    '''Pick the MR T1w from the dictionary for MR->PET registration.'''
 
     if isinstance(mri, dict):
         # check if NIfTI file is given
@@ -250,27 +265,27 @@ def pick_t1w(mri):
         elif 'T1DCM' in mri and os.path.exists(mri['MRT1W']):
             # create file name for the converted NIfTI image
             fnii = 'converted'
-            call( [rs.DCM2NIIX, '-f', fnii, mri['T1nii'] ] )
+            run( [rs.DCM2NIIX, '-f', fnii, mri['T1nii'] ] )
             ft1nii = glob.glob( os.path.join(mri['T1nii'], '*converted*.nii*') )
             ft1w = ft1nii[0]
         else:
-            print 'e> disaster: could not find a T1w image!'
-            return None
-            
+            raise IOError('could not find a T1w image!')
+
     else:
-        ('e> no correct input found for the T1w image')
-        return None
+        raise IOError('incorrect input given for the T1w image')
 
     return ft1w
 
 
-def dcminfo(dcmvar, verbose=True):
-    ''' Get basic info about the DICOM file/header.
-    '''
+def dcminfo(dcmvar, verbose=True, Cnt=None):
+    '''Get basic info about the DICOM file/header.'''
+    logger = log.info if verbose else log.debug
 
-    if isinstance(dcmvar, basestring):
-        if verbose:
-            print 'i> provided DICOM file:', dcmvar
+    if Cnt is None:
+        Cnt = {}
+
+    if isinstance(dcmvar, str):
+        logger('provided DICOM file: {}'.format(dcmvar))
         dhdr = dcm.dcmread(dcmvar)
     elif isinstance(dcmvar, dict):
         dhdr = dcmvar
@@ -278,8 +293,7 @@ def dcminfo(dcmvar, verbose=True):
         dhdr = dcmvar
 
     dtype   = dhdr[0x08, 0x08].value
-    if verbose:
-        print '   Image Type:', dtype
+    logger('   Image Type: {}'.format(dtype))
 
     #-------------------------------------------
     #> scanner ID
@@ -300,25 +314,23 @@ def dcminfo(dcmvar, verbose=True):
     csatype = ''
     if [0x29, 0x1108] in dhdr:
         csatype = dhdr[0x29, 0x1108].value
-        if verbose:
-            print '   CSA Data Type:', csatype
+        logger('   CSA Data Type: {}'.format(csatype))
 
     #> DICOM comment or on MR parameters
     cmmnt   = ''
     if [0x20, 0x4000] in dhdr:
         cmmnt = dhdr[0x0020, 0x4000].value
-        if verbose:
-            print '   Comments:', cmmnt
+        logger('   Comments: {}'.format(cmmnt))
 
-    #> MR parameters (echo time, etc) 
+    #> MR parameters (echo time, etc)
     TR = 0
     TE = 0
     if [0x18, 0x80] in dhdr:
         TR = float(dhdr[0x18, 0x80].value)
-        if verbose: print '   TR:', TR
+        logger('   TR: {}'.format(TR))
     if [0x18, 0x81] in dhdr:
         TE = float(dhdr[0x18, 0x81].value)
-        if verbose: print '   TE:', TE
+        logger('   TE: {}'.format(TE))
 
 
     #> check if it is norm file
@@ -350,14 +362,13 @@ def dcminfo(dcmvar, verbose=True):
         out = ['raw', 'physio', scanner_id]
 
     else:
-        out = ['unknown', cmmnt]
+        out = ['unknown', str(cmmnt.lower())]
 
     return out
 
 
 def list_dcm_datain(datain):
-    ''' List all DICOM file paths in the datain dictionary of input data.
-    '''
+    '''List all DICOM file paths in the datain dictionary of input data.'''
 
     if not isinstance(datain, dict):
         raise ValueError('The input is not a dictionary!')
@@ -411,68 +422,70 @@ def list_dcm_datain(datain):
     return dcmlst
 
 
-
 def dcmanonym(
         dcmpth,
         displayonly=False,
         patient='anonymised',
         physician='anonymised',
         dob='19800101',
-        verbose=True):
+        verbose=True,
+        Cnt=None):
 
-    ''' Anonymise DICOM file(s)
-        Arguments:
+    '''
+    Anonymise DICOM file(s)
+    Arguments:
         > dcmpth:   it can be passed as a single DICOM file, or
                     a folder containing DICOM files, or a list of DICOM file paths.
         > patient:  the name of the patient.
         > physician:the name of the referring physician.
         > dob:      patient's date of birth.
         > verbose:  display processing output.
+        > Cnt:      dictionary of constants (containing logging variable)
     '''
+    logger = log.info if verbose else log.debug
+
+    #> check if the dictionary of constant is given
+    if Cnt is None:
+        Cnt = {}
 
     #> check if a single DICOM file
-    if isinstance(dcmpth, basestring) and os.path.isfile(dcmpth):
+    if isinstance(dcmpth, str) and os.path.isfile(dcmpth):
         dcmlst = [dcmpth]
-        if verbose:
-            print 'i> recognised the input argument as a single DICOM file.'
+        logger('recognised the input argument as a single DICOM file.')
 
     #> check if a folder containing DICOM files
-    elif isinstance(dcmpth, basestring) and os.path.isdir(dcmpth):
+    elif isinstance(dcmpth, str) and os.path.isdir(dcmpth):
         dircontent = os.listdir(dcmpth)
         #> create a list of DICOM files inside the folder
         dcmlst = [os.path.join(dcmpth,d) for d in dircontent if os.path.isfile(os.path.join(dcmpth,d)) and d.endswith(dcmext)]
-        if verbose:
-            print 'i> recognised the input argument as the folder containing DICOM files.'
+        logger('recognised the input argument as the folder containing DICOM files.')
 
     #> check if a folder containing DICOM files
     elif isinstance(dcmpth, list):
         if not all([os.path.isfile(d) and d.endswith(dcmext) for d in dcmpth]):
             raise IOError('Not all files in the list are DICOM files.')
         dcmlst = dcmpth
-        if verbose:
-            print 'i> recognised the input argument as the list of DICOM file paths.'
+        logger('recognised the input argument as the list of DICOM file paths.')
 
     #> check if dictionary of data input <datain>
     elif isinstance(dcmpth, dict) and 'corepath' in dcmpth:
         dcmlst = list_dcm_datain(dcmpth)
-        if verbose:
-            print 'i> recognised the input argument as the dictionary of scanner data.'
+        logger('recognised the input argument as the dictionary of scanner data.')
 
     else:
         raise IOError('Unrecognised input!')
 
 
-
     for dcmf in dcmlst:
-
         #> read the file
         dhdr = dcm.dcmread(dcmf)
 
         #> get the basic info about the DICOM file
         dcmtype = dcminfo(dhdr, verbose=False)
-        if verbose:
-            print '-------------------------------'
-            print 'i> the DICOM file is for:', dcmtype
+        logger(dedent('''\
+            --------------------------------------------------
+            DICOM file is for: {}
+            --------------------------------------------------''').format(dcmtype))
 
         #> anonymise mMR data.
         if 'mmr' in dcmtype:
@@ -488,21 +501,22 @@ def dcmanonym(
 
             # string length considered for replacement
             strlen = 200
-            
+
             idx = [m.start() for m in re.finditer(r'([Pp]atients{0,1}[Nn]ame)', csa)]
-            if idx and verbose:
-                print '   > found sensitive information deep in DICOM headers:', dcmtype
+            if idx:
+                logger('DICOM> found sensitive information deep in the headers: {}'.format(dcmtype))
 
 
-            #> run the anonymisation    
+            #> run the anonymisation
             iupdate = 0
 
             for i in idx:
                 ci = i - iupdate
 
                 if displayonly:
-                    print '   > sensitive info:'
-                    print '    ', csa[ci:ci+strlen]
+                    logger(dedent('''\
+                        DICOM> sensitive info:
+                             {}''').format(csa[ci:ci+strlen]))
                     continue
 
                 rplcmnt = re.sub( r'(\{\s*\"{1,2}\W*\w+\W*\w+\W*\"{1,2}\s*\})',
@@ -511,7 +525,7 @@ def dcmanonym(
                 )
                 #> update string
                 csa = csa[:ci] + rplcmnt + csa[ci+strlen:]
-                print '   > removed sensitive information.'
+                logger('DICOM> removed sensitive information.')
                 #> correct for the number of removed letters
                 iupdate = strlen-len(rplcmnt)
 
@@ -523,40 +537,44 @@ def dcmanonym(
         #> Patient's name
         if [0x010,0x010] in dhdr:
             if displayonly:
-                print '   > sensitive info:', dhdr[0x010,0x010].name
-                print '    ', dhdr[0x010,0x010].value
+                logger(dedent('''\
+                    DICOM> sensitive info: {}
+                         > {}''').format(
+                         dhdr[0x010,0x010].name, dhdr[0x010,0x010].value))
             else:
                 dhdr[0x010,0x010].value = patient
-                if verbose: print '   > anonymised patients name'
+                logger('DICOM> anonymised patients name')
 
         #> date of birth
         if [0x010,0x030] in dhdr:
             if displayonly:
-                print '   > sensitive info:', dhdr[0x010,0x030].name
-                print '    ', dhdr[0x010,0x030].value
+                logger(dedent('''\
+                DICOM> sensitive info: {}
+                     > {}''').format(
+                     dhdr[0x010,0x030].name,  dhdr[0x010,0x030].value))
             else:
                 dhdr[0x010,0x030].value = dob
-                if verbose: print '   > anonymised date of birh'
+                logger('   > anonymised date of birth')
 
         #> physician's name
         if [0x008, 0x090] in dhdr:
             if displayonly:
-                print '   > sensitive info:',  dhdr[0x008,0x090].name
-                print '    ', dhdr[0x008,0x090].value
+                logger(dedent('''\
+                DICOM> sensitive info: {}
+                     > {}''').format(
+                     dhdr[0x008,0x090].name, dhdr[0x008,0x090].value))
             else:
                 dhdr[0x008,0x090].value = physician
-                if verbose: print '   > anonymised physician name'
+                logger('   > anonymised physician name')
 
         dhdr.save_as(dcmf)
 
 
-
-#================================================================================
-
-
-def dcmsort(folder, copy_series=False, verbose=False):
-    ''' Sort out the DICOM files in the folder according to the recorded series.
-    '''
+def dcmsort(folder, copy_series=False, verbose=False, Cnt=None):
+    '''Sort out the DICOM files in the folder according to the recorded series.'''
+    #> check if the dictionary of constant is given
+    if Cnt is None:
+        Cnt = {}
 
     # list files in the input folder
     files = os.listdir(folder)
@@ -587,16 +605,18 @@ def dcmsort(folder, copy_series=False, verbose=False):
             srs_time = dhdr[0x0008, 0x0031].value[:6]
             std_time = dhdr[0x0008, 0x0030].value[:6]
 
-            if verbose:
-                print 'series desciption:', srs_dcrp
-                print 'series time:', srs_time
-                print 'study  time:', std_time
-                print '---------------------------------------------------'
+            log.info(dedent('''\
+                --------------------------------------
+                DICOM series desciption: {}
+                DICOM series time: {}
+                DICOM study  time: {}
+                --------------------------------------''').format(
+                srs_dcrp,srs_time , std_time))
 
             #----------
             # series for any category (can be multiple scans within the same category)
             recognised_series = False
-            srs_k = srs.keys()
+            srs_k = list(srs.keys())
             for s in srs_k:
                 if  np.array_equal(srs[s]['imorient'],  ornt) and \
                     np.array_equal(srs[s]['imsize'],    imsz) and \
@@ -623,24 +643,16 @@ def dcmsort(folder, copy_series=False, verbose=False):
                 srs[s]['files'].append( os.path.join(folder, f) )
 
     return srs
-#-------------------------------------------------------------------------------
 
 
-
-
-#===============================================================================
-
-def niisort(
-        fims,
-        memlim=True
-    ):
-
-    ''' Sort all input NIfTI images and check their shape.
-        Output dictionary of image files and their properties.
-        Options:
-            memlim -- when processing large numbers of frames the memory may
-            not be large enough.  memlim causes that the output does not contain
-            all the arrays corresponding to the images.
+def niisort(fims, memlim=True):
+    '''
+    Sort all input NIfTI images and check their shape.
+    Output dictionary of image files and their properties.
+    Options:
+        memlim -- when processing large numbers of frames the memory may
+        not be large enough.  memlim causes that the output does not contain
+        all the arrays corresponding to the images.
     '''
     # number of NIfTI images in folder
     Nim = 0
@@ -650,7 +662,7 @@ def niisort(
     for f in fims:
         if f.endswith('.nii') or f.endswith('.nii.gz'):
             Nim += 1
-            _match = re.search('(?<=_frm)\d*', f)
+            _match = re.search(r'(?<=_frm)\d*', f)
             if _match:
                 frm = int(_match.group(0))
                 freelists = [frm not in l for l in sortlist]
@@ -665,47 +677,47 @@ def niisort(
     if len(sortlist)>1:
         # if more than one dynamic set is given, the dynamic mode is cancelled.
         dyn_flg = False
-        sortlist = range(Nim)
+        sortlist = list(range(Nim))
     elif len(sortlist)==1:
         dyn_flg = True
         sortlist = sortlist[0]
     else:
         raise ValueError('e> niisort input error.')
-        
-    
+
+
     # number of frames (can be larger than the # images)
     Nfrm = max(sortlist)+1
     # sort the list according to the frame numbers
     _fims = ['Blank']*Nfrm
     # list of NIfTI image shapes and data types used
     shape = []
-    dtype = []
+    datype = []
     _nii = []
     for i in range(Nim):
         if dyn_flg:
             _fims[sortlist[i]] = fims[i]
             _nii = nib.load(fims[i])
-            dtype.append(_nii.get_data_dtype()) 
+            datype.append(_nii.get_data_dtype())
             shape.append(_nii.shape)
         else:
             _fims[i] = fims[i]
             _nii = nib.load(fims[i])
-            dtype.append(_nii.get_data_dtype()) 
+            datype.append(_nii.get_data_dtype())
             shape.append(_nii.shape)
 
     # check if all images are of the same shape and data type
     if _nii and not shape.count(_nii.shape)==len(shape):
         raise ValueError('Input images are of different shapes.')
-    if _nii and not dtype.count(_nii.get_data_dtype())==len(dtype):
+    if _nii and not datype.count(_nii.get_data_dtype())==len(datype):
         raise TypeError('Input images are of different data types.')
     # image shape must be 3D
     if _nii and not len(_nii.shape)==3:
         raise ValueError('Input image(s) must be 3D.')
 
     out = {'shape':_nii.shape[::-1],
-            'files':_fims, 
+            'files':_fims,
             'sortlist':sortlist,
-            'dtype':_nii.get_data_dtype(), 
+            'dtype':_nii.get_data_dtype(),
             'N':Nim}
 
     if memlim and Nfrm>50:
@@ -726,7 +738,6 @@ def niisort(
     return out
 
 
-#================================================================================
 def dcm2nii(
         dcmpth,
         fimout = '',
@@ -737,21 +748,19 @@ def dcm2nii(
         executable = '',
         force = False,
     ):
-    ''' Convert DICOM files in folder (indicated by <dcmpth>) using DCM2NIIX
-        third-party software.
     '''
-
+    Convert DICOM files in folder (indicated by <dcmpth>) using DCM2NIIX
+    third-party software.
+    '''
     # skip conversion if the output already exists and not force is selected
     if os.path.isfile(fimout) and not force:
         return fimout
 
     if executable=='':
         try:
-            import resources
-            executable = resources.DCM2NIIX
-        except:
-            raise NameError('e> could not import resources \
-                    or find variable DCM2NIIX in resources.py')
+            executable = rs.DCM2NIIX
+        except AttributeError:
+            raise AttributeError('e> could not find variable DCM2NIIX in resources.py')
     elif not os.path.isfile(executable):
         raise IOError('e> the executable is incorrect!')
 
@@ -782,7 +791,7 @@ def dcm2nii(
 
 
     # convert the DICOM mu-map images to nii
-    call([executable, '-f', fimout, '-o', opth, dcmpth])
+    run([executable, '-f', fimout, '-o', opth, dcmpth])
 
     fniiout = glob.glob( os.path.join(opth, '*'+fimout+'*.nii*') )
 
@@ -792,24 +801,21 @@ def dcm2nii(
         raise ValueError('e> could not get the output file!')
 
 
-
-
-#================================================================================
 def dcm2im(fpth):
-    ''' Get the DICOM files from 'fpth' into an image with the affine transformation.
-        fpth can be a list of DICOM files or a path (string) to the folder with DICOM files.
     '''
-
+    Get the DICOM files from 'fpth' into an image with the affine transformation.
+    fpth can be a list of DICOM files or a path (string) to the folder with DICOM files.
+    '''
     # possible DICOM file extensions
-    ext = ('dcm', 'DCM', 'ima', 'IMA') 
-    
+    ext = ('dcm', 'DCM', 'ima', 'IMA')
+
     # case when given a folder path
-    if isinstance(fpth, basestring) and os.path.isdir(fpth):
+    if isinstance(fpth, str) and os.path.isdir(fpth):
         SZ0 = len([d for d in os.listdir(fpth) if d.endswith(ext)])
         # list of DICOM files
         fdcms = os.listdir(fpth)
         fdcms = [os.path.join(fpth,f) for f in fdcms if f.endswith(ext)]
-    
+
     # case when list of DICOM files is given
     elif isinstance(fpth, list) and os.path.isfile(os.path.join(fpth[0])):
         SZ0 = len(fpth)
@@ -820,7 +826,7 @@ def dcm2im(fpth):
         raise NameError('Unrecognised input for DICOM files.')
 
     if SZ0<1:
-        print 'e> no DICOM images in the specified path.'
+        log.error('No DICOM images in the specified path.')
         raise IOError('Input DICOM images not recognised')
 
     # pick single DICOM header
@@ -838,7 +844,7 @@ def dcm2im(fpth):
 
     #------------------------------------
     # INIT
-    # image position 
+    # image position
     P = np.zeros((SZ0,3), dtype=np.float64)
     #image orientation
     Orn = np.zeros((SZ0,6), dtype=np.float64)
@@ -863,9 +869,9 @@ def dcm2im(fpth):
             R[c,:] = np.array([float(f) for f in dhdr[0x28,0x30].value])
             S[c,:] = float(dhdr[0x18,0x50].value)
         else:
-            print 'e> could not read all the DICOM tags.'
+            log.error('could not read all the DICOM tags.')
             return {'im':[], 'affine':[], 'shape':[], 'orient':ornt, 'sruid':sruid}
-            
+
         if [0x28,0x1053] in dhdr and [0x28,0x1052] in dhdr:
             SI[c,0] = float(dhdr[0x28,0x1053].value)
             SI[c,1] = float(dhdr[0x28,0x1052].value)
@@ -875,11 +881,11 @@ def dcm2im(fpth):
 
     #check if orientation/resolution is the same for all slices
     if np.sum(Orn-Orn[0,:]) > 1e-6:
-        print 'e> varying orientation for slices'
+        log.error('varying orientation for slices')
     else:
         Orn = Orn[0,:]
     if np.sum(R-R[0,:]) > 1e-6:
-        print 'e> varying resolution for slices'
+        log.error('varying resolution for slices')
     else:
         R = R[0,:]
 
@@ -914,7 +920,7 @@ def dcm2im(fpth):
     Zz = (P[si[-1],2] - P[si[0],2])/(SZ0-1)
     Zy = (P[si[-1],1] - P[si[0],1])/(SZ0-1)
     Zx = (P[si[-1],0] - P[si[0],0])/(SZ0-1)
-    
+
 
     # dictionary for affine and image size for the image
     A = {
@@ -926,4 +932,4 @@ def dcm2im(fpth):
     }
 
     #the returned image is already scaled according to the dcm header
-    return {'im':im, 'affine':A['AFFINE'], 'shape':A['SHAPE'], 'orient':ornt, 'sruid':sruid}
\ No newline at end of file
+    return {'im':im, 'affine':A['AFFINE'], 'shape':A['SHAPE'], 'orient':ornt, 'sruid':sruid}
